@@ -6,10 +6,15 @@ import json
 import time
 import tempfile
 import math
+import asyncssh
+import os
+from random import randrange
 from dask_jobqueue.htcondor import HTCondorJob
 from subprocess import check_output, STDOUT
 from jinja2 import Environment, PackageLoader, select_autoescape
 
+from distributed.security import Security
+
 from distributed.deploy.spec import ProcessInterface, SpecCluster, NoOpAwaitable
 
 
@@ -22,19 +27,15 @@ class Process(ProcessInterface):
 
     def __init__(self, **kwargs):
         self.connection = None
-        self.connection_dash = None
-
         super().__init__(**kwargs)
 
     async def start(self):
         assert self.connection
-        assert self.connection_dash
         await super().start()
 
     async def close(self):
-        self.connection.close()
-        self.connection_dash.close()
         await super().close()
+        self.connection.close()
 
     def __repr__(self):
         return f"<SSH {type(self).__name__}: status={self.status}>"
@@ -70,8 +71,18 @@ class Scheduler(Process):
         dask.distributed.Scheduler class
     """
 
-    def __init__(self):
+    def __init__(self, sched_port=8989, dashboard_port=8787):
         self.cluster_id = None
+        self.name = os.environ.get("JUPYTERHUB_USER") + "-{}.dask-ssh".format(
+            sched_port
+        )
+        self.dash_hostname = os.environ.get(
+            "JUPYTERHUB_USER"
+        ) + "-{}.dash.dask-ssh".format(dashboard_port)
+        self.sched_port = sched_port
+        self.dash_port = dashboard_port
+
+        self.token = os.environ.get("JUPYTERHUB_API_TOKEN")
         super().__init__()
 
     def scale(self, n=0, memory=None, cores=None):
@@ -109,7 +120,14 @@ class Scheduler(Process):
             for f in files:
                 tmpl = env.get_template(f)
                 with open(tmpdirname + "/" + f, "w") as dest:
-                    dest.write(tmpl.render())
+                    dest.write(
+                        tmpl.render(
+                            name=self.name,
+                            token=self.token,
+                            sched_port=self.sched_port,
+                            dash_port=self.dash_port,
+                        )
+                    )
 
             cmd = "source ~/htc.rc; cd {}; condor_submit -spool scheduler.sub".format(
                 tmpdirname
@@ -147,28 +165,35 @@ class Scheduler(Process):
             elif job_status != 2:
                 raise Exception("Scheduler job in error {}".format(job_status))
 
-            startd_ip_classAd = classAd[0].get("StartdIpAddr")
-            if not startd_ip_classAd:
-                raise Exception("Scheduler job running but no StartdIpAddr in ClassAd")
-
-            startd_ip = startd_ip_classAd.split(":")[0].strip("<")
-
-        import asyncssh  # import now to avoid adding to module startup time
-
         self.connection = await asyncssh.connect(
-            "90.147.75.109", username="root", known_hosts=None
+            "listener.htcondor.svc.cluster.local",
+            port=8122,
+            username=self.name,
+            password=self.token,
+            known_hosts=None,
         )
-        await self.connection.forward_local_port("", 8989, startd_ip, 8989)
-
-        self.connection_dash = await asyncssh.connect(
-            "90.147.75.109", username="root", known_hosts=None
+        await self.connection.forward_local_port(
+            "127.0.0.1", self.sched_port, "127.0.0.1", self.sched_port
+        )
+        await self.connection.forward_local_port(
+            "127.0.0.1", self.dash_port, "127.0.01", self.dash_port
         )
-        await self.connection.forward_local_port("", 8787, startd_ip, 8787)
 
-        self.address = "localhost:8989"
+        self.address = "localhost:{}".format(self.sched_port)
+        self.dashboard_address = "localhost:{}".format(self.dash_port)
+
         await super().start()
 
     async def close(self):
+        from dask.distributed import Client
+
+        client = Client(address="tcp://localhost:{}".format(self.sched_port))
+
+        try:
+            client.shutdown()
+        except Exception as ex:
+            raise ex
+
         cmd = "source ~/htc.rc; condor_rm {}.0".format(self.cluster_id)
 
         try:
@@ -184,7 +209,15 @@ class Scheduler(Process):
 
 class RemoteHTCondor(SpecCluster):
     def __init__(self, asynchronous=False):
-        sched = {"cls": Scheduler, "options": {}}
+        self.sched_port = randrange(20000, 40000)
+        self.dashboard_port = randrange(20000, 40000)
+        sched = {
+            "cls": Scheduler,
+            "options": {
+                "sched_port": self.sched_port,
+                "dashboard_port": self.dashboard_port,
+            },
+        }
         super().__init__(
             scheduler=sched, asynchronous=asynchronous, workers={}, name="RemoteHTC"
         )
@@ -222,9 +255,3 @@ class RemoteHTCondor(SpecCluster):
             )
         except Exception as ex:
             raise ex
-
-
-def CreateRemoteHTCondor():
-    sched = {"cls": Scheduler, "options": {}}  # Use local scheduler for now
-
-    return SpecCluster({}, sched, name="SSHCluster")
